
package Installer;
#use strict;
use File::Copy;
#static vars&constans

my $etokenVendor = "0x0529";                       # Aladdin vendor ID
my $pcscdRestart=0;                                # Whether to restart pcscd on exit
my $action = "";                                   # current action
my $numReaders = 0;                                # Number of readers to be installed
my $maxReaders = 8;                                # Maximal number of readers supported by etokend
# List of supported devices
my @etokenDevices = ("0x050b", "0x050c", "0x0513", "0x0514", "0x0412", "0x041a", "0x0422", "0x042a");
my $IFDHandler = "aksifdh.so";                     # Name of IFD Handler
#my $libdir = "/usr/local/lib";                     # directory where IFD Handler should be located
my $pcscd_startup_item = "pcscd";                  # default

# And guess what - list of files they should be copied to
# All files are here except the IFD Handler: it is shared library and issue of
# links and versions is handled separately

#functions
#constructor
sub new 
{
    my $newObj = {};
    bless $newObj;
    return $newObj;
}

sub setAction
{
    $action = shift;
}

sub isInstall
{
    return ( $action eq "install" ) ? 1 : 0;
}

sub finita 
{
    my $retCode = shift;
    my $message = join ' ', @_;
    service ("${pcscd_startup_item}","start") if ( $pcscdRestart );
    print $message, "\n";
    exit $retCode;
}

# This function gets single parameter (process name). It returns true if process with such
# name is running, false otherwise
sub check_running
{
    return `ps ax` =~ /$_[0]/;
}

# If pcscd running - stop it and remember to restart in the future
sub stop_pcscd 
{
    $pcscdRestart = check_running ( "pcscd" );
    service ( "pcscd", "stop"); 
}

#-------------------------------------------------------------------
# This routine is intended to correct the reader.conf file
# Usage: correct_conf_file ( infile, outfile);
# Global variables - $numReaders
# The function removes all old eToken entries from reader.conf and adds the number of 
# new entries, passed as parameter. 0 means no new entries (uninstallation).

#new implementation:
#first, remove all that is related to Aladdin readers
#then, if this is an installation case, add the readers anew


sub installDLL
{
    my $libname = $_[0];
    my $linkName = "/usr/local/lib/$libname";

    if (isInstall())
    {
	opendir(DIR, ".");
	@files = grep { /$libname/ } readdir(DIR);
	closedir(DIR);
	die "shared object not found" unless ( @files );
	my $fullLibName = "/usr/local/lib/$files[0]";
	copy ( $files[0], $fullLibName );
	chmod 0755, $fullLibName;
	symlink ( $fullLibName, $linkName ) or die "can't create link $linkName: $!";
    }
    else
    {
	die "no such link: $linkName" unless -l $linkName; 
	my $fullLibName = readlink $linkName;
	unlink  $fullLibName, $linkName;
    }
}

sub modify_ld_so_conf 
{
   my $tmpfile = "/tmp/" . time;
   my $mapfile = "/etc/ld.so.conf";
   print "Modifying $mapfile\n";
   system "rm -f $tmpfile";
   correct_ld_so_conf ($mapfile, $tmpfile);
   rename $tmpfile, $mapfile;
}

#-------------------------------------------------------------------------------
# This routine is intended to add /usr/local/lib to the /etc/ld.so.conf file
# Usage: correct_ld_so_conf ( infile, outfile);
# The function adds /usr/local/lib if it is not already present in /etc/ld.so.conf
sub correct_ld_so_conf 
{

   my ($INFILE, $OUTFILE) = @_;
   my $line;
   my $usr_local_lib_present = 0;

   open(INFILE,$INFILE) or finita(1, "Unable to open file $INFILE: $!", );
   open(OUTFILE, ">" . $OUTFILE) or finita(1, "Unable to create file $OUTFILE: $!");

   while ($line = <INFILE>) {
       print OUTFILE $line;
      if ($line =~ /^\/usr\/local\/lib$/) {
	  $usr_local_lib_present = 1;
      }
   }
   close INFILE;

   if ( ! $usr_local_lib_present )
   {
       print OUTFILE "/usr/local/lib";
   }
   close OUTFILE;
}

#-----------------------------------------------------------------------

sub modify_reader_conf
{
    my $tmpFName = "/var/tmp/.tmp";
    open (OUTFILE,">$tmpFName") or finita ( 1, "Unable to create file $tmpFName: $!" );
    my $cnfFile = "/etc/reader.conf";
    if ( open (INFILE, "<$cnfFile" ) )
    {
        $insideBlock = 0; #(the one that begins with "aladdin etoken" comment and has CHANNNELID on the last line )
        while (<INFILE>)
        {
            $insideBlock = 1 if (/Aladdin/);
            print OUTFILE $_ unless ( $insideBlock || /^\s/ );
            $insideBlock = 0 if ( /CHANNELID/ );
        }
    }
    elsif ( isInstall () )#print out the header of the original reader.conf
    {
      while (<DATA>)
      {
          print OUTFILE $_;
      }
  }
    if ( isInstall () )#add actual entries for readers
    {
        my $libpath = "/usr/local/lib/aksifdh.so";
#        $libpath =~ s/\.so.*$//;
        for (my $i = 0; $i < $numReaders ; $i++)
        {
            print OUTFILE  "# Aladdin eToken virtual reader #$i\n";
	    print OUTFILE  "FRIENDLYNAME     \"AKS ifdh\"\n";
	    print OUTFILE  "DEVICENAME       /dev/null\n";
            print OUTFILE  "LIBPATH          $libpath\n";
            printf OUTFILE "CHANNELID        0x%lx\n\n", 0x11111111 + $i;
        }
    }
    close (OUTFILE);
    close (INFILE);
    move ( $tmpFName, $cnfFile ) or die "couldn't move the file $tmpFName to $cnfFile: $!";
    chmod (0777, $cnfFile );
}

#------------------------------------------------------------------------------------------

sub service 
{
   my ($service, $command) = @_;
   if ( ($command eq "start" && !check_running ( $service ) )
	|| check_running ( $service ) )
   {
       system "/etc/init.d/$service $command";
   }
}

#helper sub - creates directory recursively, like bash mkdir -p
#parameter - full path of dir to create
sub mkDirRecursive
{
    my @pathElements = split (/\//, $_[0] );
    my $path = "";
    foreach ( @pathElements )
    {
        $path = $path . $_;
        mkdir $path unless ( -d $path || $path eq "" );
        $path = $path . "/";
    }
}

#---------------------------------------------------------------
sub getDistribution
{
    open (ISSUES , "</etc/issue") or die "this program must run on linux\n";
    my @known=qw ( suse enterprise fedora );
    while ( defined ( $line =<ISSUES> ) )
    {
	next unless length $line;
	foreach $elem (@known)
	{
           if ( $line =~ /$elem/i )
	   {
	       my $platform  = $elem; 
	       close ISSUES;
	       return $platform;
	   }
	}
    }
    close ISSUES;
    die "unknown platform\n";
}

#---------------------------------------------------------------
#this is used instead of original chkconfig, in order to ensure 
#the proper order of etoken scripts
#parameters:
# service name, action (add/del), order in boot
sub chkconfigSub
{
   my ($service,$action,$order) = @_;
   my $dist = getDistribution;
   $srvScript = "/etc/init.d/$service";
   if (getDistribution eq "suse")
   {
       return;#code moved to subroutines install and uninstall
   }
   else#fedora&rhel4
   {
       for ( $rl=2; $rl<=5; $rl++ )
       {
	   my $linkName = "/etc/rc${rl}.d/S${order}etoken";
	   unlink $linkName if ( -l $linkName );
	   symlink ( "$srvScript", $linkName ) if ($action eq "add");
       }
   }
}

#---------------------------------------------------------------

sub installEtokend 
{
    if ( isInstall () )
    {
	copyExecutable ("etokend","/usr/local/sbin/etokend");
	copyExecutable ( "etoken","/etc/hotplug.d/usb/etoken.hotplug");
	copyExecutable ( "etokend.startup.script", "/etc/init.d/etokend");
	chkconfigSub ( "etokend", "add", 10 );
	service("etokend","start");
    }
    else
    {
	chkconfigSub ( "etokend", "del", 10 );
	service("etokend","stop");
	unlink "/etc/hotplug.d/usb/etoken.hotplug"
	    , "/usr/local/sbin/etokend"
	    , "/etc/init.d/etokend";
    }
}

sub prerequisites
{
   die "You must be root to run this script!\n" unless ( `whoami` eq "root\n" );
   if ( `uname` !~ /Linux/ )
   {
      die "this script must run on Linux only!\n";
   }
   die "you must install pcscd prior to eToken installation\n"  unless ( -f "/usr/local/sbin/pcscd" );
}

sub run
{
# initialization
   $pcscd_startup_item=$ENV{"PCSCD_STARTUP"} unless ( $ENV{"PCSCD_STARTUP"} eq "" );
   prerequisites();   

# check command-line
   setAction ( $ARGV[0] ) if ($#ARGV >= 0);
   if ( isInstall () ) 
   {
      $numReaders = ( $#ARGV > 0 ) ? $ARGV[1] : 1; # one more (if exists) parameter - number of readers
      $action = "" if $numReaders < 1  || $numReaders > $maxReaders;
   }
   else
   {
      $action = "" if $#ARGV != 0;  # no parameters for uninstall 
   }

   # stop modules
   if ($action eq "install") 
   {
       install ();
   }
   elsif ($action eq "uninstall")        
   {
       uninstall ();
   }
   else
   {
      print "Aladdin Etoken RTE installation/uninstallation\n";
      print "Usage: $0 {install [num_readers] | uninstall}\n";
      print "       Number of readers cannot exceed $maxReaders\n";
      exit 1;
   }

}


sub install
{
     #create log directory in case of debug version
     if ( `nm ./libetokendll.so*|grep log_context` )
    {
         Installer::mkDirRecursive ( "/var/tmp/eToken" );
         chmod (0777, '/var/tmp/eToken');
     }

     if (-f "/etc/init.d/pcscd")
     {
	 copyExecutable ( "/etc/init.d/pcscd", "/etc/init.d/pcscd.backup") 
     }
     copyExecutable ( "pcscd.startup.script", "/etc/init.d/pcscd");
     service ("pcscd", "stop" );
     installDLL ("aksifdh.so");
     modify_reader_conf();
     installEtokend ();
     service ("pcscd", "start" );
     modify_ld_so_conf;
     system "/sbin/ldconfig";
     chkconfigSub ( "pcscd", "add", 20 );
     if ( getDistribution eq "suse" )
     {
	 copyExecutable ( "etstart.suse", "/etc/init.d/etstart" );
         system "/usr/lib/lsb/install_initd /etc/init.d/etstart";
    }
     print "Aladdin Etoken RTE installation finished\n";
}

sub verifyInstalled
{
    die "eToken RTE doesn't seem to be installed on this machine\n" 
	unless ( -f "/usr/local/sbin/etokend" or -f "/usr/local/sbin/etsrvd" );
}

sub uninstall
{
      verifyInstalled();
      service ("pcscd","stop");
      unlink ("/etc/init.d/pcscd");
      if ( -f "/etc/init.d/pcscd.backup")
      {
	  move ( "/etc/init.d/pcscd.backup","/etc/init.d/pcscd" )
      }
      else
      {
	  chkconfigSub ( "pcscd", "del", 20 );
      }
      installEtokend ();
      modify_reader_conf ();
      installDLL ("aksifdh.so");
      #remove logs
      system ( 'rm -rf /var/tmp/eToken' ) if -d '/var/tmp/eToken';
      #remove pcscd tmp files, in case they left somehow
      unlink ( "/var/run/pcscd.pub" ) if ( -f "/var/run/pcscd.pub" );
      unlink ( "/var/run/pcscd.comm" ) if ( -f "/var/run/pcscd.comm" );
     if ( getDistribution eq "suse" )
     {
         system "/usr/lib/lsb/remove_initd /etc/init.d/etstart";
	 unlink ( "/etc/init.d/etstart" );
     }
      print "Aladdin Etoken RTE uninstallation finished\n";
}


sub checkInstallations 
{
      -f "/usr/local/sbin/pcscd" or -f "/usr/sbin/pcscd" or die "PC/SC-Lite should be installed before RTE distribution\n";
      -f "/usr/local/sbin/pcscd" and -f "/usr/sbin/pcscd" 
	  and print "Warning: you have two pcscd installations (in /usr and in /usr/local)\n";
}

#parameters:
#1. src file name
#2. destination file name with full path
sub copyExecutable
{
    die "file $_[0] doesn't exist!" unless ( -f $_[0] );
    my $dirName = $_[1];
    $dirName =~ s/(.*)\/\w+.*$/$1/;#remove the file name-leave just the dir name
    mkDirRecursive ( $dirName );
    copy ( $_[0],$_[1] ) or die "can't copy $_[0]. $!";
    chmod ( 0777, $_[1] );
}



__END__
# Configuration file for IFD Handler 2.0 drivers
# FRIENDLYNAME: Any name
# DEVICENAME: Any name
# LIBPATH: Location of the driver library for your reader
# CHANNELID:
#             0x000001 - COM1
#             0x000002 - COM2
#             0x000003 - COM3
#             0x000004 - COM4

